@@ -23,6 +23,7 @@
 #include <QVBoxLayout>
 #include <QDragEnterEvent>
 #include <QMouseEvent>
+#include <QScrollArea>
 
 #include "installer.h"
 
@@ -112,7 +113,6 @@ SplashInstaller::SplashInstaller (QWidget *aParent, const char *aName, bool aIni
 
   QHBoxLayout* hbox = new QHBoxLayout( this );
   hbox->setMargin( 0 );
-  hbox->setSpacing( KDialog::spacingHint() );
 
   QVBoxLayout* leftbox = new QVBoxLayout(  );
   hbox->addItem( leftbox );
@@ -140,13 +140,14 @@ SplashInstaller::SplashInstaller (QWidget *aParent, const char *aName, bool aIni
   mBtnRemove = new KPushButton( KIcon("edit-delete"), i18n("Remove Theme"), this );
   mBtnRemove->setToolTip(i18n("Remove the selected theme from your disk"));
   mBtnRemove->setWhatsThis(i18n("This will remove the selected theme from your disk."));
+  mBtnRemove->setEnabled( false );
   leftbox->addWidget( mBtnRemove );
   connect(mBtnRemove, SIGNAL(clicked()), SLOT(slotRemove()));
 
   mBtnTest = new KPushButton( KIcon("document-preview"), i18n("Test Theme"), this );
   mBtnTest->setToolTip(i18n("Test the selected theme"));
   mBtnTest->setWhatsThis(i18n("This will test the selected theme."));
-
+  mBtnTest->setEnabled( false );
   leftbox->addWidget( mBtnTest );
   connect(mBtnTest, SIGNAL(clicked()), SLOT(slotTest()));
 
@@ -155,21 +156,25 @@ SplashInstaller::SplashInstaller (QWidget *aParent, const char *aName, bool aIni
   rightbox->setSpacing( KDialog::spacingHint() );
   hbox->setStretchFactor( rightbox, 3 );
 
+  QScrollArea* scrollarea = new QScrollArea(this);
+  scrollarea->setAlignment(Qt::AlignHCenter|Qt::AlignVCenter);
   mPreview = new QLabel(this);
+  scrollarea->setWidget(mPreview);
   mPreview->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
   mPreview->setFrameStyle(QFrame::Panel|QFrame::Sunken);
   mPreview->setMinimumSize(QSize(320,240));
   mPreview->setAlignment(Qt::AlignHCenter|Qt::AlignVCenter);
-  rightbox->addWidget(mPreview);
-  rightbox->setStretchFactor( mPreview, 3 );
+  rightbox->addWidget(scrollarea);
+  rightbox->setStretchFactor( scrollarea, 3 );
 
   mText = new QTextEdit(this);
   mText->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Preferred );
-  mText->setMinimumSize(mText->sizeHint());
+  mText->setMinimumSize(mText->sizeHint().width(), 7 * mText->fontMetrics().height());
   mText->setReadOnly(true);
   rightbox->addWidget(mText);
   rightbox->setStretchFactor( mText, 1 );
 
+
   readThemesList();
   load();
 }
@@ -200,6 +205,7 @@ void SplashInstaller::addNewTheme(const KUrl &srcURL)
   QString dir = KGlobal::dirs()->saveLocation("ksplashthemes");
 
   KUrl url;
+  QStringList themeNames;
   QString filename = srcURL.fileName();
   int i = filename.lastIndexOf('.');
   // Convert extension to lower case.
@@ -211,49 +217,64 @@ void SplashInstaller::addNewTheme(const KUrl &srcURL)
   if ( KIO::NetAccess::exists( url, KIO::NetAccess::SourceSide, 0 ) )
     KIO::NetAccess::del( url, 0 );
 
-  bool rc = KIO::NetAccess::file_copy(srcURL, url, 0);
-  if (!rc)
+  if (srcURL.fileName().toLower() == "theme.rc" ) // uncompressed theme selected
   {
-    kWarning() << "Failed to copy theme " << srcURL.fileName()
-        << " into temporary directory " << url.path() << endl;
-    return;
-  }
+    QString themeName;
+    // Get the name of the Theme from the theme.rc file
+    KConfig _cnf(srcURL.path());
+    KConfigGroup cnf(&_cnf, QString("KSplash Theme: %1").arg(themeName) );
 
-  // Extract into theme directory: we may have multiple themes in one tarball!
-  KTar tarFile(url.path());
-  if (!tarFile.open(QIODevice::ReadOnly))
-  {
-    kWarning() << "Unable to open archive: " << url.path();
-    KIO::NetAccess::del( url, 0 );
-    return;
-  }
-  KArchiveDirectory const *ad = tarFile.directory();
+    // copy directory of theme.rc to ThemesDir
+    KIO::NetAccess::dircopy(KUrl(srcURL.directory()), KUrl(dir + themeName));
 
-  // Find first directory entry.
-  QStringList entries = ad->entries();
-  QStringList themeNames;
-  foreach(QString s, entries)
+    themeNames << themeName;
+  }
+  else
   {
-    if( ad->entry(s)->isDirectory() )
+    bool rc = KIO::NetAccess::file_copy(srcURL, url, 0);
+    if (!rc)
     {
-      // each directory may contain one theme
-      themeNames << s;
+      kWarning() << "Failed to copy theme " << srcURL.fileName()
+	  << " into temporary directory " << url.path() << endl;
+      return;
     }
-  }
-  if (themeNames.count() < 1)
-  {
-    kWarning() << "No directory in archive: " << url.path();
+
+    // Extract into theme directory: we may have multiple themes in one tarball!
+    KTar tarFile(url.path());
+    if (!tarFile.open(QIODevice::ReadOnly))
+    {
+      kWarning() << "Unable to open archive: " << url.path();
+      KIO::NetAccess::del( url, 0 );
+      return;
+    }
+    KArchiveDirectory const *ad = tarFile.directory();
+
+    // Find first directory entry.
+    QStringList entries = ad->entries();
+    foreach(const QString& s, entries)
+    {
+      if( ad->entry(s)->isDirectory() )
+      {
+	// each directory may contain one theme
+	themeNames << s;
+      }
+    }
+    if (themeNames.count() < 1)
+    {
+      kWarning() << "No directory in archive: " << url.path();
+      tarFile.close();
+      KIO::NetAccess::del( url, 0 );
+      return;
+    }
+
+    // copy the theme into the "ksplashthemes" directory
+    ad->copyTo(dir);
+
     tarFile.close();
     KIO::NetAccess::del( url, 0 );
-    return;
+
   }
 
-  // copy the theme into the "ksplashthemes" directory
-  ad->copyTo(dir);
-
-  tarFile.close();
-  KIO::NetAccess::del( url, 0 );
-
   readThemesList();
   mThemesList->setCurrentRow(findTheme(themeNames.first()));
   if (mThemesList->currentItem())
@@ -273,7 +294,7 @@ void SplashInstaller::readThemesList()
   QDir dir;
   QStringList subdirs;
   QStringList::ConstIterator name;
-  for(name = entryList.begin(); name != entryList.end(); name++)
+  for(name = entryList.constBegin(); name != entryList.constEnd(); name++)
   {
     dir = *name;
     if (!dir.exists())
@@ -281,7 +302,7 @@ void SplashInstaller::readThemesList()
     subdirs = dir.entryList( QDir::Dirs );
     // kDebug() << "readThemesList: " << subdirs;
     // TODO: Make sure it contains a *.rc file.
-    for (QStringList::Iterator l = subdirs.begin(); l != subdirs.end(); l++ )
+    for (QStringList::const_iterator l = subdirs.constBegin(); l != subdirs.constEnd(); l++ )
       if ( !(*l).startsWith(QString(".")) )
       {
         mThemesList->blockSignals( true ); // Don't activate any theme until all themes are loaded.
@@ -404,15 +425,15 @@ void SplashInstaller::slotSetTheme(int id)
         // Get theme information.
         infoTxt = "<qt>";
         if ( cnf.hasKey( "Name" ) )
-          infoTxt += i18n( "<b>Name:</b> %1", cnf.readEntry( "Name", i18n( "Unknown" ) ) ) + "<br />";
+          infoTxt += i18n( "<b>Name:</b> %1", cnf.readEntry( "Name", i18nc( "Unknown name", "Unknown" ) ) ) + "<br />";
         if ( cnf.hasKey( "Description" ) )
-          infoTxt += i18n( "<b>Description:</b> %1", cnf.readEntry( "Description", i18n( "Unknown" ) ) ) + "<br />";
+          infoTxt += i18n( "<b>Description:</b> %1", cnf.readEntry( "Description", i18nc( "Unknown description", "Unknown" ) ) ) + "<br />";
         if ( cnf.hasKey( "Version" ) )
-          infoTxt += i18n( "<b>Version:</b> %1", cnf.readEntry( "Version", i18n( "Unknown" ) ) ) + "<br />";
+          infoTxt += i18n( "<b>Version:</b> %1", cnf.readEntry( "Version", i18nc( "Unknown version", "Unknown" ) ) ) + "<br />";
         if ( cnf.hasKey( "Author" ) )
-          infoTxt += i18n( "<b>Author:</b> %1", cnf.readEntry( "Author", i18n( "Unknown" ) ) ) + "<br />";
+          infoTxt += i18n( "<b>Author:</b> %1", cnf.readEntry( "Author", i18nc( "Unknown author", "Unknown" ) ) ) + "<br />";
         if ( cnf.hasKey( "Homepage" ) )
-          infoTxt += i18n( "<b>Homepage:</b> %1", cnf.readEntry( "Homepage", i18n( "Unknown" ) ) ) + "<br />";
+          infoTxt += i18n( "<b>Homepage:</b> %1", cnf.readEntry( "Homepage", i18nc( "Unknown homepage", "Unknown" ) ) ) + "<br />";
         infoTxt += "</qt>";
 
         QString pluginName( cnf.readEntry( "Engine", "KSplashX" ).trimmed() );
@@ -473,7 +494,7 @@ void SplashInstaller::slotAdd()
   static QString path;
   if (path.isEmpty()) path = QDir::homePath();
 
-  KFileDialog dlg(path, "*.tgz *.tar.gz *.tar.bz2|" + i18n( "KSplash Theme Files" ), this);
+  KFileDialog dlg(path, "*.tgz *.tar.gz *.tar.bz2 theme.rc|" + i18n( "KSplash Theme Files" ), this);
   dlg.setCaption(i18n("Add Theme"));
   if (!dlg.exec())
     return;
@@ -527,7 +548,7 @@ void SplashInstaller::slotTest()
     KProcess proc;
     proc << "ksplashsimple" << themeName << "--test";
     if (proc.execute())
-      KMessageBox::error(this,i18n("Unable to start ksplashsimple."));
+      KMessageBox::error(this,i18n("Failed to successfully test the splash screen."));
     return;
   }
   else if( mEngineOfSelected == "KSplashX" )
@@ -535,7 +556,7 @@ void SplashInstaller::slotTest()
     KProcess proc;
     proc << "ksplashx" << themeName << "--test";
     if (proc.execute())
-      KMessageBox::error(this,i18n("Unable to start ksplashx."));
+      KMessageBox::error(this,i18n("Failed to successfully test the splash screen."));
     return;
   }
   else // KSplashML engines
@@ -543,7 +564,7 @@ void SplashInstaller::slotTest()
     KProcess proc;
     proc << "ksplash" << "--test" << "--theme" << themeName;
     if (proc.execute())
-      KMessageBox::error(this,i18n("Unable to start ksplash."));
+      KMessageBox::error(this,i18n("Failed to successfully test the splash screen."));
   }
 }
 
