@@ -31,6 +31,12 @@ MakeTransparentEffect::MakeTransparentEffect()
     : fadeout( NULL )
     , current( NULL )
     , previous( NULL )
+    {
+    reconfigure( ReconfigureAll );
+    active = effects->activeWindow();
+    }
+
+void MakeTransparentEffect::reconfigure( ReconfigureFlags )
     {
     KConfigGroup conf = effects->effectConfig("MakeTransparent");
     decoration = conf.readEntry( "Decoration", 1.0 );
@@ -52,11 +58,13 @@ MakeTransparentEffect::MakeTransparentEffect()
         popupmenus = menus;
         tornoffmenus = menus;
         }
-    active = effects->activeWindow();
     moveresize_timeline.setCurveShape( TimeLine::EaseOutCurve );
-    moveresize_timeline.setDuration( conf.readEntry( "Duration", 800 ) );
+    moveresize_timeline.setDuration( animationTime( conf, "Duration", 800 ) );
     activeinactive_timeline.setCurveShape( TimeLine::EaseInOutCurve );
-    activeinactive_timeline.setDuration( conf.readEntry( "Duration", 800 ) );
+    activeinactive_timeline.setDuration( animationTime( conf, "Duration", 800 ) );
+
+    // Repaint the screen just in case the user changed the inactive opacity
+    effects->addRepaintFull();
     }
 
 void MakeTransparentEffect::prePaintWindow( EffectWindow* w, WindowPrePaintData& data, int time )
@@ -70,6 +78,8 @@ void MakeTransparentEffect::prePaintWindow( EffectWindow* w, WindowPrePaintData&
         // don't clear PAINT_WINDOW_OPAQUE, contents are not affected
         data.clip &= w->contentsRect().translated( w->pos()); // decoration cannot clip
         }
+    if( inactive != 1.0 && isInactive( w ))
+        data.setTranslucent();
     if(( moveresize != 1.0 && ( w->isUserMove() || w->isUserResize()))
         || ( dialogs != 1.0 && w->isDialog()))
         {
@@ -156,7 +166,7 @@ void MakeTransparentEffect::paintWindow( EffectWindow* w, int mask, QRegion regi
                 }
             }
 
-        // Menues and combos
+        // Menus and combos
         if( dropdownmenus != 1.0 && w->isDropdownMenu() )
             data.opacity *= dropdownmenus;
         if( popupmenus != 1.0 && w->isPopupMenu() )
