@@ -59,7 +59,7 @@ namespace KWin
 
 WinInfo::WinInfo( Client * c, Display * display, Window window,
     Window rwin, const unsigned long pr[], int pr_size )
-    : NETWinInfo( display, window, rwin, pr, pr_size, NET::WindowManager ), m_client( c )
+    : NETWinInfo2( display, window, rwin, pr, pr_size, NET::WindowManager ), m_client( c )
     {
     }
 
@@ -68,6 +68,11 @@ void WinInfo::changeDesktop(int desktop)
     m_client->workspace()->sendClientToDesktop( m_client, desktop, true );
     }
 
+void WinInfo::changeFullscreenMonitors( NETFullscreenMonitors topology )
+    {
+    m_client->updateFullscreenMonitors( topology );
+    }
+
 void WinInfo::changeState( unsigned long state, unsigned long mask )
     {
     mask &= ~NET::Sticky; // KWin doesn't support large desktops, ignore
@@ -279,6 +284,10 @@ bool Workspace::workspaceEvent( XEvent * e )
                 return true;
                 }
             break;
+        case ConfigureNotify:
+            if( e->xconfigure.event == rootWindow())
+                x_stacking_dirty = true;
+            break;
         };
 
     if( Client* c = findClient( WindowMatchPredicate( e->xany.window )))
@@ -484,6 +493,7 @@ bool Workspace::workspaceEvent( XEvent * e )
                     addRepaintFull();
                     QTimer::singleShot( 2000, this, SLOT( addRepaintFull()));
                     }
+                checkCompositeTimer();
                 }
             break;
         default:
@@ -617,7 +627,7 @@ bool Client::windowEvent( XEvent* e )
                 }
             else
                 { // forward to the frame if there's possibly another compositing manager running
-                NETWinInfo i( display(), frameId(), rootWindow(), 0 );
+                NETWinInfo2 i( display(), frameId(), rootWindow(), 0 );
                 i.setOpacity( info->opacity());
                 }
             }
@@ -750,29 +760,17 @@ bool Client::mapRequestEvent( XMapRequestEvent* e )
         }
     if( isTopMenu() && workspace()->managingTopMenus())
         return true; // kwin controls these
-    switch ( mappingState() )
+    // also copied in clientMessage()
+    if( isMinimized())
+        unminimize();
+    if( isShade())
+        setShade( ShadeNone );
+    if( !isOnCurrentDesktop())
         {
-        case WithdrawnState:
-            assert( false ); // WMs are not supposed to manage clients in Withdrawn state,
-//        manage();      // after initial mapping manage() is called from createClient()
-            break;
-        case IconicState:
-	// also copied in clientMessage()
-            if( isMinimized())
-                unminimize();
-            if( isShade())
-                setShade( ShadeNone );
-            if( !isOnCurrentDesktop())
-                {
-                if( workspace()->allowClientActivation( this ))
-                    workspace()->activateClient( this );
-                else
-                    demandAttention();
-                }
-            break;
-        case NormalState:
-	    // TODO fake MapNotify?
-            break;
+        if( workspace()->allowClientActivation( this ))
+            workspace()->activateClient( this );
+        else
+            demandAttention();
         }
     return true;
     }
@@ -792,25 +790,7 @@ void Client::unmapNotifyEvent( XUnmapEvent* e )
         if( ignore )
             return;
         }
-    switch( mappingState())
-        {
-        case IconicState:
-            releaseWindow();
-          return;
-        case NormalState:
-            // maybe we will be destroyed soon. Check this first.
-            XEvent ev;
-            if( XCheckTypedWindowEvent (display(), window(),
-                DestroyNotify, &ev) ) // TODO I don't like this much
-                {
-                destroyClient(); // deletes this
-                return;
-                }
-            releaseWindow();
-          break;
-    default:
-        assert( false );
-        }
+    releaseWindow();
     }
 
 void Client::destroyNotifyEvent( XDestroyWindowEvent* e )
@@ -960,13 +940,16 @@ void Client::enterNotifyEvent( XCrossingEvent* e )
              e->mode == NotifyUngrab ) ) 
         {
 
-        if (options->shadeHover && isShade()) 
+        if ( options->shadeHover )
             {
-            delete shadeHoverTimer;
-            shadeHoverTimer = new QTimer( this );
-            connect( shadeHoverTimer, SIGNAL( timeout() ), this, SLOT( shadeHover() ));
-            shadeHoverTimer->setSingleShot( true );
-            shadeHoverTimer->start( options->shadeHoverInterval );
+            cancelShadeHoverTimer();
+            if (isShade())
+              {
+              shadeHoverTimer = new QTimer( this );
+              connect( shadeHoverTimer, SIGNAL( timeout() ), this, SLOT( shadeHover() ));
+              shadeHoverTimer->setSingleShot( true );
+              shadeHoverTimer->start( options->shadeHoverInterval );
+              }
             }
 
         if ( options->focusPolicy == Options::ClickToFocus )
@@ -974,7 +957,8 @@ void Client::enterNotifyEvent( XCrossingEvent* e )
 
         if ( options->autoRaise && !isDesktop() &&
              !isDock() && !isTopMenu() && workspace()->focusChangeEnabled() &&
-             workspace()->topClientOnDesktop( workspace()->currentDesktop()) != this ) 
+             workspace()->topClientOnDesktop( workspace()->currentDesktop(),
+                 options->separateScreenFocus ? screen() : -1 ) != this ) 
             {
             delete autoRaiseTimer;
             autoRaiseTimer = new QTimer( this );
@@ -1032,9 +1016,14 @@ void Client::leaveNotifyEvent( XCrossingEvent* e )
             {
             cancelAutoRaise();
             workspace()->cancelDelayFocus();
-            cancelShadeHover();
+            cancelShadeHoverTimer();
             if ( shade_mode == ShadeHover && !moveResizeMode && !buttonDown )
-               setShade( ShadeNormal );
+              {
+              shadeHoverTimer = new QTimer( this );
+              connect( shadeHoverTimer, SIGNAL( timeout() ), this, SLOT( shadeUnhover() ));
+              shadeHoverTimer->setSingleShot( true );
+              shadeHoverTimer->start( options->shadeHoverInterval );
+              }
             }
         if ( options->focusPolicy == Options::FocusStrictlyUnderMouse )
             if ( isActive() && lostMouse )
@@ -1093,7 +1082,7 @@ void Client::updateMouseGrab()
         {
         XUngrabButton( display(), AnyButton, AnyModifier, wrapperId());
         // keep grab for the simple click without modifiers if needed (see below)
-        bool not_obscured = workspace()->topClientOnDesktop( workspace()->currentDesktop(), true, false ) == this;
+        bool not_obscured = workspace()->topClientOnDesktop( workspace()->currentDesktop(), -1, true, false ) == this;
         if( !( !options->clickRaise || not_obscured ))
             grabButton( None );
         return;
@@ -1109,7 +1098,7 @@ void Client::updateMouseGrab()
         // is unobscured or if the user doesn't want click raise
         // (it is unobscured if it the topmost in the unconstrained stacking order, i.e. it is
         // the most recently raised window)
-        bool not_obscured = workspace()->topClientOnDesktop( workspace()->currentDesktop(), true, false ) == this;
+        bool not_obscured = workspace()->topClientOnDesktop( workspace()->currentDesktop(), -1, true, false ) == this;
         if( !options->clickRaise || not_obscured )
             ungrabButton( None );
         else
@@ -1325,7 +1314,7 @@ void Client::processMousePressEvent( QMouseEvent* e )
     {
     if( e->type() != QEvent::MouseButtonPress )
         {
-        kWarning() << "processMousePressEvent()" ;
+        kWarning(1212) << "processMousePressEvent()" ;
         return;
         }
     int button;
@@ -1706,6 +1695,7 @@ void Unmanaged::configureNotifyEvent( XConfigureEvent* e )
         addWorkspaceRepaint( geometry()); // damage old area
         QRect old = geom;
         geom = newgeom;
+        addRepaintFull();
         if( old.size() != geom.size())
             discardWindowPixmap();
         if( scene != NULL )
