@@ -20,26 +20,27 @@
 #ifndef TIMEENGINE_H
 #define TIMEENGINE_H
 
-#include "plasma/dataengine.h"
-
-class QTimer;
+#include <Plasma/DataEngine>
 
 /**
- * This class evaluates the basic expressions given in the interface.
+ * This engine provides the current date and time for a given
+ * timezone.
+ *
+ * "Local" is a special source that is an alias for the current
+ * timezone.
  */
 class TimeEngine : public Plasma::DataEngine
 {
     Q_OBJECT
 
     public:
-        TimeEngine(QObject* parent, const QVariantList& args);
-        ~TimeEngine();
+        TimeEngine(QObject *parent, const QVariantList &args);
+        void init();
+        QStringList sources() const;
 
     protected:
         bool sourceRequestEvent(const QString &name);
-        bool updateSourceEvent(const QString& source);
+        bool updateSourceEvent(const QString &source);
 };
 
-K_EXPORT_PLASMA_DATAENGINE(time, TimeEngine)
-
-#endif
+#endif // TIMEENGINE_H
