@@ -203,13 +203,18 @@ PanelView::PanelView(Plasma::Containment *panel, int id, QWidget *parent)
       m_triggerEntered(false)
 {
     Q_ASSERT(qobject_cast<Plasma::Corona*>(panel->scene()));
+
+    m_strutsTimer = new QTimer(this);
+    m_strutsTimer->setSingleShot(true);
+    connect(m_strutsTimer, SIGNAL(timeout()), this, SLOT(updateStruts()));
+
     if (panel) {
         connect(panel, SIGNAL(destroyed(QObject*)), this, SLOT(panelDeleted()));
         connect(panel, SIGNAL(toolBoxToggled()), this, SLOT(togglePanelController()));
         kDebug() << "Panel geometry is" << panel->geometry();
     }
 
-    connect(this, SIGNAL(sceneRectAboutToChange()), this, SLOT(updatePanelGeometry()));
+    connect(this, SIGNAL(sceneRectAboutToChange()), this, SLOT(pinchContainmentToCurrentScreen()));
 
     // Graphics view setup
     setFrameStyle(QFrame::NoFrame);
@@ -248,10 +253,6 @@ PanelView::PanelView(Plasma::Containment *panel, int id, QWidget *parent)
     KWindowSystem::setOnAllDesktops(winId(), true);
 
     QTimer::singleShot(0, this, SLOT(init()));
-
-    m_strutsTimer = new QTimer(this);
-    m_strutsTimer->setSingleShot(true);
-    connect(m_strutsTimer,SIGNAL(timeout()),this,SLOT(updateStruts()));
 }
 
 PanelView::~PanelView()
@@ -276,6 +277,10 @@ void PanelView::init()
             this, SLOT(pinchContainmentToCurrentScreen()));
     connect(screens, SIGNAL(screenMoved(Kephal::Screen *, QPoint, QPoint)),
             this, SLOT(updatePanelGeometry()));
+    connect(screens, SIGNAL(screenAdded(Kephal::Screen *)),
+            this, SLOT(updateStruts()));
+    connect(screens, SIGNAL(screenRemoved(int)),
+            this, SLOT(updateStruts()));
 }
 
 void PanelView::setLocation(Plasma::Location location)
@@ -326,7 +331,7 @@ void PanelView::setLocation(Plasma::Location location)
     }
 
     //kDebug() << "!!!!!!!!!!!!!!!!!! about to set to" << location << panelHeight << formFactor;
-    disconnect(this, SIGNAL(sceneRectAboutToChange()), this, SLOT(updatePanelGeometry()));
+    disconnect(this, SIGNAL(sceneRectAboutToChange()), this, SLOT(pinchContainmentToCurrentScreen()));
     c->setFormFactor(formFactor);
     c->setLocation(location);
 
@@ -342,7 +347,7 @@ void PanelView::setLocation(Plasma::Location location)
     pinchContainment(screenRect);
     KWindowSystem::setOnAllDesktops(winId(), true);
     //updatePanelGeometry();
-    connect(this, SIGNAL(sceneRectAboutToChange()), this, SLOT(updatePanelGeometry()));
+    connect(this, SIGNAL(sceneRectAboutToChange()), this, SLOT(pinchContainmentToCurrentScreen()));
 }
 
 Plasma::Location PanelView::location() const
@@ -420,9 +425,6 @@ void PanelView::updatePanelGeometry()
     switch (location()) {
     case Plasma::TopEdge:
     case Plasma::BottomEdge:
-        if (m_alignment != Qt::AlignCenter) {
-            m_offset = qMax(m_offset, 0);
-        }
         //resize the panel if is too large
         if (geom.width() > screenGeom.width()) {
             geom.setWidth(screenGeom.width());
@@ -448,9 +450,6 @@ void PanelView::updatePanelGeometry()
 
     case Plasma::LeftEdge:
     case Plasma::RightEdge:
-        if (m_alignment != Qt::AlignCenter) {
-            m_offset = qMax(m_offset, 0);
-        }
         //resize the panel if is too tall
         if (geom.height() > screenGeom.height()) {
             geom.setHeight(screenGeom.height());
@@ -651,6 +650,12 @@ void PanelView::pinchContainment(const QRect &screenGeom)
             }
         }
 
+        // if the maximum-size is 0, set it to 100%
+        if (max.width() <= 0) {
+            c->setMaximumSize(sw, max.height());
+            max = c->maximumSize();
+        }
+
         if (m_offset + max.width() > sw) {
             //kDebug() << "max size is too wide!";
             if (max.width() > sw) {
@@ -669,6 +674,12 @@ void PanelView::pinchContainment(const QRect &screenGeom)
             }
         }
 
+        // if the maximum-size is 0, set it to 100%
+        if (max.height() <= 0) {
+            c->setMaximumSize(max.width(), sh);
+            max = c->maximumSize();
+        }
+
         if (m_offset + max.height() > sh) {
             //kDebug() << "max size is too tall!";
             if (max.height() > sh) {
@@ -679,6 +690,12 @@ void PanelView::pinchContainment(const QRect &screenGeom)
         }
     }
 
+    // resize to max if for some reason the size is empty
+    // otherwise its not possible to interact with the panel at all
+    if (c->size().isEmpty()) {
+        c->resize(max);
+    }
+
     if (m_lastHorizontal != horizontal ||
         m_lastSeenSize != (horizontal ? sw : sh)) {
         m_lastHorizontal = horizontal;
@@ -1117,19 +1134,16 @@ void PanelView::leaveEvent(QEvent *event)
             createUnhideTrigger();
         }
     } else if (m_visibilityMode == AutoHide && !m_editting) {
-        // try not to hide if we have an associated popup or window about
-        if (hasPopup()) {
-            // don't hide yet, but start polling to see when we should
-            if (!m_mousePollTimer) {
-                m_mousePollTimer = new QTimer(this);
-            }
-
-            disconnect(m_mousePollTimer, SIGNAL(timeout()), this, SLOT(hideMousePoll()));
-            connect(m_mousePollTimer, SIGNAL(timeout()), this, SLOT(hideMousePoll()));
-            m_mousePollTimer->start(200);
-        } else {
-            startAutoHide();
+        // even if we dont have a popup, we'll start a timer, so
+        // that the panel stays if the mouse only leaves for a
+        // few ms
+        if (!m_mousePollTimer) {
+            m_mousePollTimer = new QTimer(this);
         }
+
+        disconnect(m_mousePollTimer, SIGNAL(timeout()), this, SLOT(hideMousePoll()));
+        connect(m_mousePollTimer, SIGNAL(timeout()), this, SLOT(hideMousePoll()));
+        m_mousePollTimer->start(200);
     }
 
     Plasma::View::leaveEvent(event);
