@@ -188,7 +188,6 @@ PowerDevilDaemon::PowerDevilDaemon(QObject *parent, const QList<QVariant>&)
                                    const char*, const QString&)),
             SLOT(emitCriticalNotification(const QString&, const QString&,
                                           const char*, const QString&)));
-    connect(d->ksmServerIface, SIGNAL(loggingOut(int,int,int)), this, SLOT(loggingOut(int,int,int)));
 
     /* Time for setting up polling! We can have different methods, so
      * let's check what we got.
@@ -218,7 +217,7 @@ PowerDevilDaemon::PowerDevilDaemon(QObject *parent, const QList<QVariant>&)
 
     // This gets registered to avoid double copies.
     QDBusConnection::sessionBus().registerService("org.kde.powerdevilsystem");
-
+    conn.interface()->registerService("org.freedesktop.Policy.Power");
     // All systems up Houston, let's go!
     refreshStatus();
 }
@@ -233,11 +232,6 @@ void PowerDevilDaemon::batteryRemainingTimeChanged(int time)
     kDebug() << KGlobal::locale()->formatDuration(time);
 }
 
-void PowerDevilDaemon::loggingOut(int confirm, int sdtype, int sdmode)
-{
-    d->lockHandler->inhibit("","");
-}
-
 SuspensionLockHandler *PowerDevilDaemon::lockHandler()
 {
     return d->lockHandler;
